#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: Seaky
# @Date:   2019/4/29 22:43

__title__ = 'seakylib'
__author__ = 'Seaky'
__version__ = '0.0.2'
__copyright__ = 'Copyright (c) 2019 Seaky'
